WEAKNESS_BEFORE_STOP_WORDS = r'(,|) ((has|have|contain[s]?|introduc(es|ed|ing)|suffers from) (an |a |)|(was|were) (found|discovered) to (contain|have) (a|an) )'
WEAKNESS_PATTERN = r'(?P<weakness>.*?)'
WEAKNESS_AFTER_STOP_WORDS = r'( ((V|v)ulnerabilit(y|ies)|issue[s]?)( (was|were) (discovered|found)|)| exists|)'

COMPONENT_BEFORE_STOP_WORDS = r' (in (the |its |)|(starting |)at )'
COMPONENT_PATTERN = r'(?P<component>.+?)'
COMPONENT_AFTER_STOP_WORDS = r'(file|component|call|feature|function|method|process|implementation|console)[s]?'

QUOTES_PATTERN = r'[`\'"\u2018\u2019\u201C\u201D]'
DATE_VERSION_PATTERN = r'(before|through) \d{4}-\d{2}-\d{2}'
YEAR_PATTERN = r'(?:200\d|201\d|202\d)'
VERSION_PATTERN = r'\b(?:v\.|v|V|R)*(?:\d+(?:\.(?:\d+|x))+)(?:[a-zA-Z\-]+\d*)*\b'
COMMIT_VERSION_PATTERN = r'\b[0-9a-f]{5,40}\b'

CONFIGURATION_BEFORE_STOP_WORDS = r' (in|within|of|on) (some |the |a |)'
CONFIGURATION_PATTERN = (r'(?P<configuration>[a-zA-Z0-9-\/\(\),]{2,}.+?(?:' + VERSION_PATTERN + r'|' + COMMIT_VERSION_PATTERN
                         + r'|' + DATE_VERSION_PATTERN + r'|' + YEAR_PATTERN + r')(?: and (?:before|prior|(possibly |)earlier|below|up to and including)| plugin|.+?version[s]?|))')
CONFIGURATION_AFTER_STOP_WORDS = r'( for(?: [a-zA-Z0-9\!\-\.]+){1,4}|)'

ATTACKER_BEFORE_STOP_WORDS = r'(,|) (could |which (could |)|may |might |that (can |)|)(allow|enabl[e]?|(exploitable|used) by)(s|ed|ing|) (a |an |any |)'
ATTACKER_PATTERN = r'(?P<attacker>.*?(attacker|user|server|administrator|admin|subscriber|adversary|person)[s]?)'
ATTACKER_AFTER_STOP_WORDS = r'(,|)( with.+?(permission|privilege|role|access|right)[s]?( and above| as low.+?|)| (((who are |)able|with the ability) to).+?| such as.+?| permitted| on the .+?|)'

ATTACK_BEFORE_STOP_WORDS = r' (to (perform|execute) )'
ATTACK_PATTERN = r'(?P<attack>.*?(attack)[s]?)'

CONDITION_BEFORE_STOP_WORDS = r'(,|) (if |where |(even |)when |before using )(the |a |)'
CONDITION_PATTERN = r'(?P<condition>.+?)'

IMPACT_ACTIONS = [
    "cause", "bypass", "gain", "hijack", "escalate", "inject", "trigger", "upload", "update", "connect", "send",
    "redirect", "replace", "change", "create", "read", "enumerate", "impersonate", "call", "request", "delete", "steal",
    "define", "lead[s]?", "check", "access", "execute(d|)", "run", "force", "crash", "overwrite", "corrupt(ion|ed|)",
    "write", "take", "make", "conduct", "perform", "obtain", "achieve", "remotely", "arbitrarily", "trick", "copy",
    "replace", "accept", "disable", "initiate", "rebuild", "include", "modify", "add", "edit", "configure", "have",
    "guess", "view", "list", "retrieve", "insert", "disclose", "reset", "craft", "open", "discover", "determine",
    "visit", "introduce", "alter", "harvest", "exfiltrate", "clear", "supply", "leak", "use", "implement", "remote",
    "compromise", "launch", "interact", "exploit", "denial", "enable", "command", "memory", "set", "arbitrary"
]

IMPACT_ACTIONS_PATTERN = '|'.join(IMPACT_ACTIONS)

IMPACT_BEFORE_STOP_WORDS = (
    r'(?:,?\s*(?:to(?: be| potentially)?|that|(lead|result)(s|ing|ed|) (to|in)\s*(a|an|the|)|causing a|could cause( a|)|'
    r'(and|which)(?: can| could| may)?)\s+)'
)

IMPACT_PATTERN = (
    rf'(?P<impact>(?:{IMPACT_ACTIONS_PATTERN})\b.+?)'
)

VECTOR_BEFORE_STOP_WORDS = r'(,|) (via|(by |)using|(as demonstrated |)by) (an |a |vectors |)'
VECTOR_PATTERN = r'(?P<vector>.+?)'

ROOT_CAUSE_BEFORE_STOP_WORD = r'(,|) ((which (is|are)|does) not|because( of| a| the|)|(due|fail) to|insecurely|(that|) improperly|uses|causes|omits) '
ROOT_CAUSE_PATTERN = r'(?P<root_cause>.+?)'


FUNCTION_NAME_PATTERN = r'[a-zA-Z_][a-zA-Z0-9_:\(\)]+'
EXTENSION_PATTERN = r"\.[a-zA-Z0-9]{1,4}"
DIR_FILE_NAME_PATTERN = r"[a-zA-Z0-9_-]+"
FILE_PATTERN = DIR_FILE_NAME_PATTERN + EXTENSION_PATTERN
RELATIVE_PATH_PATTERN = DIR_FILE_NAME_PATTERN + r"(?:/(?:" + DIR_FILE_NAME_PATTERN + r"))+" + EXTENSION_PATTERN


REPLACEMENT_PHRASES = {
    r'via the (' + FUNCTION_NAME_PATTERN + r') function in the component (.+?)\.$': r'in the \1 function at \2 component.',
    r'via the (' + FUNCTION_NAME_PATTERN + r') (function|component) (at|in) (.+?)\.$': r'in the \1 at \4.',
    r'via the (function|component) (' + FUNCTION_NAME_PATTERN + r') (at|in) (.+?)\.$': r'in the \2 \1 at \4.',
    r'via (' + FUNCTION_NAME_PATTERN + r') class in (.+?)\.$': r'in the \1 at \2.',
    r'via the (' + FUNCTION_NAME_PATTERN + r') function.$': r'in the \1 function.',
    r'via the function (' + FUNCTION_NAME_PATTERN + r').$': r'in the \1 function.',
    r'via the component (.+?)\.$': r'in the \1.',
    r'in function (' + FUNCTION_NAME_PATTERN + r') in (.+?) in ': r'in \1 function at \2 in ',
    r'in function (' + FUNCTION_NAME_PATTERN + r') in ' + CONFIGURATION_PATTERN: r'in \1 function in \2',
    r'at (.+?) in (' + FUNCTION_NAME_PATTERN + r') in ' + CONFIGURATION_PATTERN: r'in \2 at \1 in \3',
    r'out of bounds': r'out-of-bounds',
    r'has been identified in': r'exists in',
    r'in the context of': r'within',
    r'that( can|) result[s]?( in|)': r'resulting in',
    r'which may lead to': r'leads to',
    r'^There is a': r'A',
    r' to to ': ' to ',
    r',(?=\S)': ', ', # fixes commas missing a space after them
    r'in ' + QUOTES_PATTERN + r'([a-zA-Z0-9-\/\(\),]{2,})' + QUOTES_PATTERN: r'in \1',
    r' <= (' + VERSION_PATTERN + ')': r' up to and including \1',
    r'is vulnerable to': r'has',
    r'may be vulnerable to': r"could cause",
    r' attackers to Injecting ': ' attackers to inject ',
    r'that could be leveraged to cause': 'could cause',
    r'which can be used by an attacker': 'allows an attacker',
    r'(leading to|resulting in) (.*?) (vulnerability|issue)': r'introducing \2 \3',
    r'(is|are) affected by (.*?) vulnerability': r'has \2 vulnerability',
    r'(\d+), ([a-z]+)': r'\1 \2',
    r'^In (applications using |)' + CONFIGURATION_PATTERN + r'(,|) there is a': r'\2 has a',
    r'^(?:In |)' + CONFIGURATION_PATTERN + r' ((?:a |).+?)(?: vulnerability|) (exists|was found) in': r'\1 has \4 in',
    r'^In ' + CONFIGURATION_PATTERN + r' (.*?) vulnerability exists': r'\1 has \4',
    r'^(' + FUNCTION_NAME_PATTERN + r') (in) (' + RELATIVE_PATH_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 \2 \3 file \4 ',
    r'(' + FUNCTION_NAME_PATTERN + r') function in (' + RELATIVE_PATH_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 function at \2 file \3 ',
    r'(' + FUNCTION_NAME_PATTERN + r') function in (' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 function at \2 file \3 ',
    r'(' + FUNCTION_NAME_PATTERN + r') (in) (' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS : r'\1 at \3 file \4 ',
    r'^(' + RELATIVE_PATH_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS: r'\1 file \2 ',
    r'^(' + FILE_PATTERN + r')' + CONFIGURATION_BEFORE_STOP_WORDS: r'\1 file \2 '
}

NOISE_PATTERNS = [
    r'\s+\(.*?\)',
    r'\s+\[CWE-\d{1,4}\]',
    r'( of type|)\s+CWE-\d{1,4}',
    r' a different (?:vulnerability|vector) than (CVE-\d{4}-\d{4,7}(?:,\s*CVE-\d{4}-\d{4,7})*)',
    r' a related issue to CVE-\d{4}-\d{4,7}',
    r' and CVE-\d{4}-\d{4,7}',
    r' fixed in ' + VERSION_PATTERN,
    r'can be abused '
]



from ...common import (CONFIGURATION_PATTERN, ATTACKER_PATTERN, ATTACKER_BEFORE_STOP_WORDS, IMPACT_PATTERN,
                       IMPACT_BEFORE_STOP_WORDS, ROOT_CAUSE_PATTERN, ROOT_CAUSE_BEFORE_STOP_WORD, ATTACK_PATTERN,
                       ATTACK_BEFORE_STOP_WORDS, VECTOR_BEFORE_STOP_WORDS, VECTOR_PATTERN, CONDITION_BEFORE_STOP_WORDS,
                       CONDITION_PATTERN, ATTACKER_AFTER_STOP_WORDS, COMPONENT_BEFORE_STOP_WORDS, COMPONENT_PATTERN,
                       WEAKNESS_BEFORE_STOP_WORDS, WEAKNESS_PATTERN)

# configuration root_cause attacker
CONFIG_CAUSE_ATTACKER_PATTERN = (r'(?:The |)' + CONFIGURATION_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD +
                                 ROOT_CAUSE_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
                                 ATTACKER_AFTER_STOP_WORDS)

# configuration root_cause attacker impact condition weakness
DETAILS_6_1 = (CONFIG_CAUSE_ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN +
               CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + WEAKNESS_BEFORE_STOP_WORDS + WEAKNESS_PATTERN + r'\.$')

# configuration root_cause attacker attack vector
DETAILS_5_1 = (CONFIG_CAUSE_ATTACKER_PATTERN + ATTACK_BEFORE_STOP_WORDS + ATTACK_PATTERN +
               VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$')

# configuration root_cause attacker impact vector
DETAILS_5_2 = (CONFIG_CAUSE_ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN +
               VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$')

# configuration root_cause attacker attack condition
DETAILS_5_3 = (CONFIG_CAUSE_ATTACKER_PATTERN + ATTACK_BEFORE_STOP_WORDS + ATTACK_PATTERN +
               CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + r'\.$')

# configuration root_cause attacker impact condition
DETAILS_5_4 = (CONFIG_CAUSE_ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN +
               CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + r'\.$')

# configuration root_cause attacker attack
DETAILS_4_1 = CONFIG_CAUSE_ATTACKER_PATTERN + ATTACK_BEFORE_STOP_WORDS + ATTACK_PATTERN + r'\.$'

# configuration root_cause attacker impact
DETAILS_4_2 = CONFIG_CAUSE_ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.$'


CONFIG_CAUSE_ATTACKER_PATTERNS = {
    '6_1': DETAILS_6_1,
    '5_1': DETAILS_5_1,
    '5_2': DETAILS_5_2,
    '5_3': DETAILS_5_3,
    '5_4': DETAILS_5_4,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2
}



from enum import Enum


class DetailType(Enum):
    """
        Enum for starting details type
    """
    CONFIG_WEAKNESS_COMP = "config_weakness_comp"
    CONFIG_WEAKNESS_ATTACKER = "config_weakness_attacker"
    CONFIG_WEAKNESS_CONDITION = "config_weakness_condition"
    CONFIG_ATTACKER_IMPACT = "config_attacker_impact"
    CONFIG_CAUSE_ATTACKER = "config_cause_attacker"
    CONFIG_CAUSE_WEAKNESS = "config_cause_weakness"
    CONFIG_CONDITION_CAUSE = "config_condition_cause"
    CONFIG_CAUSE_CONDITION = "config_cause_condition"
    WEAKNESS_COMP_CONFIG = "weakness_comp_config"
    WEAKNESS_CONFIG_COMP = "weakness_config_comp"
    WEAKNESS_CONFIG_ATTACKER = "weakness_config_attacker"
    WEAKNESS_CONFIG_VECTOR = "weakness_config_vector"
    WEAKNESS_CONFIG_CAUSE = "weakness_config_cause"
    WEAKNESS_CONFIG_CONDITION = "weakness_config_condition"
    COMPONENT_CONFIG_WEAKNESS = "comp_config_weakness"
    COMPONENT_CONFIG_ATTACKER = "comp_config_attacker"
    COMPONENT_CONFIG_CAUSE = "comp_config_cause"
    COMPONENT_CONFIG_CONDITION = "comp_config_condition"



from ..common.enums import DetailType

from .weakness.configuration.attacker import WEAKNESS_CONFIG_ATTACKER_PATTERN, WEAKNESS_CONFIG_ATTACKER_PATTERNS
from .weakness.configuration.component import WEAKNESS_CONFIG_COMP_PATTERN, WEAKNESS_CONFIG_COMP_PATTERNS
from .weakness.configuration.vector import WEAKNESS_CONFIG_VECTOR_PATTERN, WEAKNESS_CONFIG_VECTOR_PATTERNS
from .weakness.component.configuration import WEAKNESS_COMP_CONFIG_PATTERN, WEAKNESS_COMP_CONFIG_PATTERNS
from .weakness.configuration.cause import WEAKNESS_CONFIG_CAUSE_PATTERN, WEAKNESS_CONFIG_CAUSE_PATTERNS
from .weakness.configuration.condition import WEAKNESS_CONFIG_CONDITION_PATTERN, WEAKNESS_CONFIG_CONDITION_PATTERNS

from .configuration.weakness.component import CONFIG_WEAKNESS_COMP_PATTERN, CONFIG_WEAKNESS_COMP_PATTERNS
from .configuration.weakness.attacker import CONFIG_WEAKNESS_ATTACKER_PATTERN, CONFIG_WEAKNESS_ATTACKER_PATTERNS
from .configuration.weakness.condition import CONFIG_WEAKNESS_CONDITION_PATTERN, CONFIG_WEAKNESS_CONDITION_PATTERNS
from .configuration.attacker.impact import CONFIG_ATTACKER_IMPACT_PATTERN, CONFIG_ATTACKER_IMPACT_PATTERNS
from .configuration.cause.attacker import CONFIG_CAUSE_ATTACKER_PATTERN, CONFIG_CAUSE_ATTACKER_PATTERNS
from .configuration.cause.weakness import CONFIG_CAUSE_WEAKNESS_PATTERN, CONFIG_CAUSE_WEAKNESS_PATTERNS
from .configuration.cause.condition import CONFIG_CAUSE_CONDITION_PATTERN, CONFIG_CAUSE_CONDITION_PATTERNS
from .configuration.condition.cause import CONFIG_CONDITION_CAUSE_PATTERN, CONFIG_CONDITION_CAUSE_PATTERNS

from .component.configuration.weakness import COMPONENT_CONFIG_WEAKNESS_PATTERN, COMPONENT_CONFIG_WEAKNESS_PATTERNS
from .component.configuration.attacker import COMPONENT_CONFIG_ATTACKER_PATTERN, COMPONENT_CONFIG_ATTACKER_PATTERNS
from .component.configuration.cause import COMPONENT_CONFIG_CAUSE_PATTERN, COMPONENT_CONFIG_CAUSE_PATTERNS
from .component.configuration.condition import COMPONENT_CONFIG_CONDITION_PATTERN, COMPONENT_CONFIG_CONDITION_PATTERNS


STARTING_DETAILS_MAPPING = {
    DetailType.COMPONENT_CONFIG_WEAKNESS: COMPONENT_CONFIG_WEAKNESS_PATTERN,
    DetailType.CONFIG_WEAKNESS_COMP: CONFIG_WEAKNESS_COMP_PATTERN,
    DetailType.CONFIG_WEAKNESS_ATTACKER: CONFIG_WEAKNESS_ATTACKER_PATTERN,
    DetailType.CONFIG_WEAKNESS_CONDITION: CONFIG_WEAKNESS_CONDITION_PATTERN,
    DetailType.WEAKNESS_COMP_CONFIG: WEAKNESS_COMP_CONFIG_PATTERN,
    DetailType.WEAKNESS_CONFIG_COMP: WEAKNESS_CONFIG_COMP_PATTERN,
    DetailType.COMPONENT_CONFIG_CAUSE: COMPONENT_CONFIG_CAUSE_PATTERN,
    DetailType.COMPONENT_CONFIG_CONDITION: COMPONENT_CONFIG_CONDITION_PATTERN,
    DetailType.COMPONENT_CONFIG_ATTACKER: COMPONENT_CONFIG_ATTACKER_PATTERN,
    DetailType.WEAKNESS_CONFIG_ATTACKER: WEAKNESS_CONFIG_ATTACKER_PATTERN,
    DetailType.WEAKNESS_CONFIG_VECTOR: WEAKNESS_CONFIG_VECTOR_PATTERN,
    DetailType.WEAKNESS_CONFIG_CAUSE: WEAKNESS_CONFIG_CAUSE_PATTERN,
    DetailType.WEAKNESS_CONFIG_CONDITION: WEAKNESS_CONFIG_CONDITION_PATTERN,
    DetailType.CONFIG_ATTACKER_IMPACT: CONFIG_ATTACKER_IMPACT_PATTERN,
    DetailType.CONFIG_CAUSE_ATTACKER: CONFIG_CAUSE_ATTACKER_PATTERN,
    DetailType.CONFIG_CAUSE_WEAKNESS: CONFIG_CAUSE_WEAKNESS_PATTERN,
    DetailType.CONFIG_CONDITION_CAUSE: CONFIG_CONDITION_CAUSE_PATTERN,
    DetailType.CONFIG_CAUSE_CONDITION: CONFIG_CAUSE_CONDITION_PATTERN
}

DETAILS_PATTERNS_MAPPING = {
    DetailType.COMPONENT_CONFIG_WEAKNESS: COMPONENT_CONFIG_WEAKNESS_PATTERNS,
    DetailType.CONFIG_WEAKNESS_COMP: CONFIG_WEAKNESS_COMP_PATTERNS,
    DetailType.CONFIG_WEAKNESS_ATTACKER: CONFIG_WEAKNESS_ATTACKER_PATTERNS,
    DetailType.CONFIG_WEAKNESS_CONDITION: CONFIG_WEAKNESS_CONDITION_PATTERNS,
    DetailType.WEAKNESS_COMP_CONFIG: WEAKNESS_COMP_CONFIG_PATTERNS,
    DetailType.WEAKNESS_CONFIG_COMP: WEAKNESS_CONFIG_COMP_PATTERNS,
    DetailType.COMPONENT_CONFIG_CAUSE: COMPONENT_CONFIG_CAUSE_PATTERNS,
    DetailType.COMPONENT_CONFIG_CONDITION: COMPONENT_CONFIG_CONDITION_PATTERNS,
    DetailType.COMPONENT_CONFIG_ATTACKER: COMPONENT_CONFIG_ATTACKER_PATTERNS,
    DetailType.WEAKNESS_CONFIG_ATTACKER: WEAKNESS_CONFIG_ATTACKER_PATTERNS,
    DetailType.WEAKNESS_CONFIG_VECTOR: WEAKNESS_CONFIG_VECTOR_PATTERNS,
    DetailType.WEAKNESS_CONFIG_CAUSE: WEAKNESS_CONFIG_CAUSE_PATTERNS,
    DetailType.WEAKNESS_CONFIG_CONDITION: WEAKNESS_CONFIG_CONDITION_PATTERNS,
    DetailType.CONFIG_ATTACKER_IMPACT: CONFIG_ATTACKER_IMPACT_PATTERNS,
    DetailType.CONFIG_CAUSE_ATTACKER: CONFIG_CAUSE_ATTACKER_PATTERNS,
    DetailType.CONFIG_CAUSE_WEAKNESS: CONFIG_CAUSE_WEAKNESS_PATTERNS,
    DetailType.CONFIG_CONDITION_CAUSE: CONFIG_CONDITION_CAUSE_PATTERNS,
    DetailType.CONFIG_CAUSE_CONDITION: CONFIG_CAUSE_CONDITION_PATTERNS
}



from ...common import (CONFIGURATION_PATTERN, ROOT_CAUSE_PATTERN, ROOT_CAUSE_BEFORE_STOP_WORD,
                       CONDITION_BEFORE_STOP_WORDS, CONDITION_PATTERN, IMPACT_BEFORE_STOP_WORDS, IMPACT_PATTERN,
                       ATTACKER_BEFORE_STOP_WORDS, ATTACKER_PATTERN, ATTACKER_AFTER_STOP_WORDS,
                       VECTOR_BEFORE_STOP_WORDS, VECTOR_PATTERN, WEAKNESS_BEFORE_STOP_WORDS, WEAKNESS_PATTERN)


# configuration root_cause condition
CONFIG_CAUSE_CONDITION_PATTERN = (r'(?:The |)' + CONFIGURATION_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD +
                                 ROOT_CAUSE_PATTERN + CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN)

# configuration root_cause condition attacker impact vector
#DETAILS_6_1 = (CONFIG_CAUSE_CONDITION_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
#               ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS +
#               VECTOR_PATTERN + r'\.$')

# configuration root_cause condition attacker impact vector
#DETAILS_6_1 = (CONFIG_CAUSE_CONDITION_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
#               ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS +
#               VECTOR_PATTERN + r'\.$')

# configuration root_cause condition attacker impact
#DETAILS_5_1 = (CONFIG_CAUSE_CONDITION_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN +
#               ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.$')

# configuration root_cause condition impact
DETAILS_4_1 = CONFIG_CAUSE_CONDITION_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.$'

# configuration root_cause condition weakness
DETAILS_4_2 = CONFIG_CAUSE_CONDITION_PATTERN + WEAKNESS_BEFORE_STOP_WORDS + WEAKNESS_PATTERN + r'\.$'


CONFIG_CAUSE_CONDITION_PATTERNS = {
#    '6_1': DETAILS_6_1,
#    '5_1': DETAILS_5_1,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2,
}



from ...common import (CONFIGURATION_PATTERN, COMPONENT_PATTERN, CONFIGURATION_BEFORE_STOP_WORDS,
                       COMPONENT_AFTER_STOP_WORDS, ATTACKER_BEFORE_STOP_WORDS,
                       ATTACKER_PATTERN, ATTACKER_AFTER_STOP_WORDS, IMPACT_BEFORE_STOP_WORDS, IMPACT_PATTERN,
                       VECTOR_BEFORE_STOP_WORDS, VECTOR_PATTERN, ROOT_CAUSE_BEFORE_STOP_WORD, ROOT_CAUSE_PATTERN,
                       CONDITION_BEFORE_STOP_WORDS, CONDITION_PATTERN)


# component config condition
COMPONENT_CONFIG_CONDITION_PATTERN = (
        r'(?:The |)' + COMPONENT_PATTERN + COMPONENT_AFTER_STOP_WORDS +  CONFIGURATION_BEFORE_STOP_WORDS +
        CONFIGURATION_PATTERN + CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN
)

# component config condition root_cause attacker impact vector
DETAILS_7_1 = (COMPONENT_CONFIG_CONDITION_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN +
               ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$')


COMPONENT_CONFIG_CONDITION_PATTERNS = {
    '7_1': DETAILS_7_1
}



import json

from pathlib import Path
from cve_driller.driller import CVEDriller


if __name__ == "__main__":
    driller = CVEDriller(verbose=True)
    cve_id = "CVE-2018-7187"

    input_path = f"data/inputs/CWE-78/comp_config_condition/{cve_id}.txt"
    #description = open(input_path, 'r').read()
    description = "A Server-side Template Injection (SSTI) vulnerability exists in Nystudio107 Seomatic 3.4.12 in src/helpers/UrlHelper.php via the host header."
    detail_type, details = driller(description)
    # CVE-2021-44618

    if details:
        print(json.dumps(details, indent=4))
    exit()
    input_path = Path("data/inputs")
    output_path = Path("data/outputs")

    dataset_path = Path("data/dataset.jsonl")

    with dataset_path.open(mode="a") as df:
        for cwe in input_path.iterdir():
            if not cwe.is_dir():
                continue

            for detail_type in cwe.iterdir():
                if not detail_type.is_dir():
                    continue

                for filename in detail_type.iterdir():
                    if filename.suffix != ".txt":
                        print(f"Skipping {filename}, does not have a .txt extension")
                        continue

                    with filename.open(mode="r") as f:
                        prompt = f.read()

                    output_file = output_path / cwe.name / detail_type.name / f"{filename.stem}.json"

                    if not output_file.exists():
                        print(f"Skipping {filename}, does not exist")
                        continue

                    with output_file.open(mode="r") as f:
                        completion = json.load(f)

                    jsonl = {"messages": [
                        {"role": "user", "content": prompt},
                        {"role": "assistant", "content": json.dumps(completion)}]
                    }

                    df.write(json.dumps(jsonl) + "\n")

    #if details:
    #    output_path = f"data/outputs/{detail_type.value}/{cve_id}.json"

    #    with open(output_path, 'w') as f:
    #        json.dump(details, f, indent=4)



from ...common import (WEAKNESS_PATTERN, WEAKNESS_AFTER_STOP_WORDS, CONFIGURATION_PATTERN, ATTACKER_PATTERN,
                       CONFIGURATION_BEFORE_STOP_WORDS, COMPONENT_PATTERN, COMPONENT_BEFORE_STOP_WORDS,
                       ATTACKER_BEFORE_STOP_WORDS, IMPACT_PATTERN, IMPACT_BEFORE_STOP_WORDS, ATTACKER_AFTER_STOP_WORDS,
                       ROOT_CAUSE_BEFORE_STOP_WORD, ROOT_CAUSE_PATTERN, CONDITION_BEFORE_STOP_WORDS, CONDITION_PATTERN,
                       VECTOR_BEFORE_STOP_WORDS, VECTOR_PATTERN)

# weakness configuration component
WEAKNESS_CONFIG_COMP_PATTERN = (r'(An |A |)' + WEAKNESS_PATTERN + WEAKNESS_AFTER_STOP_WORDS +
                                CONFIGURATION_BEFORE_STOP_WORDS + CONFIGURATION_PATTERN + COMPONENT_BEFORE_STOP_WORDS +
                                COMPONENT_PATTERN)

# weakness configuration component attacker impact root_cause
DETAILS_6_1 = (WEAKNESS_CONFIG_COMP_PATTERN +
               ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.$')

# weakness configuration component attacker impact root_cause
DETAILS_6_2 = (WEAKNESS_CONFIG_COMP_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + ROOT_CAUSE_BEFORE_STOP_WORD + ROOT_CAUSE_PATTERN + r'\.$')

# weakness configuration component attacker impact condition
DETAILS_6_3 = (WEAKNESS_CONFIG_COMP_PATTERN +
               ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + ATTACKER_AFTER_STOP_WORDS + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + r'\.$')

# weakness configuration component attacker impact condition
DETAILS_6_4 = (WEAKNESS_CONFIG_COMP_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + r'\.$')

# weakness configuration component attacker impact
DETAILS_5_1 = (WEAKNESS_CONFIG_COMP_PATTERN + ATTACKER_BEFORE_STOP_WORDS + ATTACKER_PATTERN + IMPACT_BEFORE_STOP_WORDS +
               IMPACT_PATTERN + r'\.$')

# weakness configuration component impact
DETAILS_4_1 = WEAKNESS_CONFIG_COMP_PATTERN + IMPACT_BEFORE_STOP_WORDS + IMPACT_PATTERN + r'\.$'

# weakness configuration component vector
DETAILS_4_2 = WEAKNESS_CONFIG_COMP_PATTERN + VECTOR_BEFORE_STOP_WORDS + VECTOR_PATTERN + r'\.$'

# weakness configuration component condition
DETAILS_4_3 = WEAKNESS_CONFIG_COMP_PATTERN + CONDITION_BEFORE_STOP_WORDS + CONDITION_PATTERN + r'\.$'


WEAKNESS_CONFIG_COMP_PATTERNS = {
    '6_1': DETAILS_6_1,
    '6_2': DETAILS_6_2,
    '6_3': DETAILS_6_3,
    '6_4': DETAILS_6_4,
    '5_1': DETAILS_5_1,
    '4_1': DETAILS_4_1,
    '4_2': DETAILS_4_2,
    '4_3': DETAILS_4_3
}
